from pymongo import MongoClient
import configreader as cfgr


def getMongoConn(filePath):

    dataconfig = cfgr.dataconfigread(filePath)
    mongoClient = MongoClient(dataconfig['mongodb']['url'])
    db = mongoClient[dataconfig['mongodb']['database']]
    collection = db[dataconfig['mongodb']['collection']]

    return collection




from classify import classify
import numpy as np


def getData(client, action, filePath):

    inputs, outputs, timestamps = classify(client, filePath)

    if action == "train":
        inp = []
        outp = []

        for input in inputs:
            for i in range(0, len(inputs[input])):
                inp.append(inputs[input][i])

        for output in outputs:
            for i in range(0, len(outputs[output])):
                outp.append(outputs[output][i])

        inp = np.array(inp)
        outp = np.array(outp)

        inp_max, inp = normalize(inp)
        outp_max, outp = normalize(outp)

        return inp, outp, timestamps

    if action == "predict":

        inp = {}
        outp = {}
        inp_max = {}
        outp_max = {}

        for input in inputs:
            i = inputs[input]
            i = np.array(i)
            i_max, i = normalize(i)
            inp_max[input] = i_max
            inp[input] = i

        for output in outputs:
            o = outputs[output]
            o = np.array(o)
            o_max, o = normalize(o)
            outp_max[output] = o_max
            outp[output] = o

        return inp_max, inp, outp_max, outp, timestamps


def normalize(data):

    maximums = np.amax(abs(data), axis=0)

    if hasattr(maximums, "__len__"):
        for i in range (0, len(maximums)):
            if maximums[i]==0 :
                maximums[i]=1
        return maximums, data/maximums
    else:
        if maximums == 0:
            maximums = 1
        return maximums, data/maximums


import tensorflow as tf
from tensorflow import keras

def buildModel(input_train, output_train, modelPath):

    activation = "sigmoid"
    dropout = 0
    optimizer = "adam"
    loss = "mean_squared_error"
    epoch = 10

    initializer = tf.keras.initializers.RandomNormal(mean=0., stddev=1.)

    if len(input_train.shape)>1:
        input_size = input_train.shape[1]
    else:
        input_size = 1

    if len(output_train.shape)>1:
        output_size = output_train.shape[1]
    else:
        output_size = 1

    model = keras.Sequential([
        keras.layers.Dense(input_size, activation=activation, kernel_initializer=initializer),
        keras.layers.Dropout(dropout),
        keras.layers.Dense(250, activation=activation, kernel_initializer=initializer),
        keras.layers.Dropout(dropout),
        keras.layers.Dense(250, activation=activation, kernel_initializer=initializer),
        keras.layers.Dropout(dropout),
        keras.layers.Dense(output_size, activation=activation, kernel_initializer=initializer)
    ])

    if optimizer == 'sgd':
        optimizer = tf.keras.optimizers.SGD(learning_rate=0.01, momentum=0.0, nesterov=True)
    elif optimizer == 'adam':
        optimizer = tf.keras.optimizers.Adam(learning_rate=0.001, beta_1=0.9, beta_2=0.999, epsilon=1e-07, amsgrad=True)
    elif optimizer == 'adadelta':
        optimizer = tf.keras.optimizers.Adadelta(learning_rate=0.001, rho=0.95, epsilon=1e-07)
    elif optimizer == 'adagrad':
        optimizer = tf.keras.optimizers.Adagrad(learning_rate=0.001, initial_accumulator_value=0.1, epsilon=1e-07)
    elif optimizer == 'adamax':
        optimizer = tf.keras.optimizers.Adamax(learning_rate=0.001, beta_1=0.9, beta_2=0.999, epsilon=1e-07)
    elif optimizer == 'ftrl':
        optimizer = tf.keras.optimizers.Ftrl(learning_rate=0.001, learning_rate_power=-0.5,
                                             initial_accumulator_value=0.1, l1_regularization_strength=0.0,
                                             l2_regularization_strength=0.0, l2_shrinkage_regularization_strength=0.0)
    elif optimizer == 'nadam':
        optimizer = tf.keras.optimizers.Nadam(learning_rate=0.001, beta_1=0.9, beta_2=0.999, epsilon=1e-07)
    elif optimizer == 'rmsprop':
        optimizer = tf.keras.optimizers.RMSprop(learning_rate=0.001, rho=0.9, momentum=0.0, epsilon=1e-07,
                                                centered=True)

    model.compile(optimizer=optimizer,
                  loss=loss,
                  metrics=['accuracy'])

    model.save(modelPath, model.fit(input_train, output_train, epochs=epoch))


import logging

from average import getAverage
from datasource import getData
from model import buildModel
from prediction import getPrediction
import math


class LogAnalysis:

    def traincontroller(self, client, filePath, modelPath):

        try:
            input_data, output_data, timestamp = getData(client, "train", filePath)
            buildModel(input_data, output_data, modelPath)
            return True
        except Exception as e:
            logging.exception(e)
            return False


    def predictcontroller(self, client, filePath, modelPath):

        try:
            input_max, inputs, output_max, outputs, timestamps = getData(client, "predict", filePath)

            predictions = {}

            for key in inputs:
                input = inputs[key]
                output = outputs[key]
                predict = getPrediction(input, output, modelPath)
                predict_list = []
                for i in range(0, len(predict)):
                    predict_list.append(math.floor(predict[i][0] + 0.5))
                predictions[key] = predict_list

            averages = getAverage(predictions, timestamps)

            return averages
        except Exception as e:
            logging.exception(e)
            return None


from elasticsearch import Elasticsearch
import mongoconn as mc


def getlogs(client, filePath):

    collection = mc.getMongoConn(filePath)
    log_details = collection.find({'client': client}, {'_id': 0})
    log_details = list(log_details)

    es_hosts = log_details[0]['hosts'].split(',')

    es_client = Elasticsearch(hosts=es_hosts)
    all_indices = es_client.indices.get_alias('*')

    indices = []
    alllogs = {}

    for index in all_indices:
        if "logstash" in index:
            indices.append(index)

    for i in range (0, len(indices)):
        logs = []
        data = es_client.search(index=indices[i], body = {
                    'size' : 10000,
                    'query': {
                        'match_all' : {}
                    }
                })

        hits = data['hits']['hits']

        for hit in hits:
            logs.append({'timestamp': hit['_source']['@timestamp'], 'message': hit['_source']['message']})

        alllogs[indices[i]] = logs

    return alllogs


from . import loganalysis, average, classify, configreader, datasource, elasticsearchconn, model, mongoconn, prediction

__all__ = [
    'average',
    'classify',
    'configreader',
    'datasource',
    'elasticsearchconn',
    'loganalysis',
    'model',
    'mongoconn',
    'prediction'
]



from tensorflow.keras.models import load_model

def getPrediction(input_test, output_test, modelPath):

    model = load_model(modelPath)

    test_loss, test_acc = model.evaluate(input_test, output_test, verbose=2)

    print('\nTest accuracy:', test_acc)

    predictions = model.predict(input_test)

    return predictions


from datetime import datetime


def getAverage(outputs, timestamps):

    counts = {}
    for i in range (0, 24):
        counts[str(i)] = 0

    for log in outputs:
        counts = getCount(outputs[log], timestamps[log], counts)

    for count in counts:
        counts[count] = counts[count]/len(timestamps)

    return counts

def getCount(logs, timestamp, counts):

    for i in range (0, len(timestamp)):
        timestamp[i] = timestamp[i][:-5]
        timestamp[i] = datetime.strptime(timestamp[i], "%Y-%m-%dT%H:%M:%S")

        if(logs[i]==1):
            counts[str(timestamp[i].hour)] = counts[str(timestamp[i].hour)] + 1

    return counts



from setuptools import setup, find_packages

with open('README.md') as readme_file:
    README = readme_file.read()

with open('HISTORY.md') as history_file:
    HISTORY = history_file.read()
	
setup_args = dict(
    name='loganalysispackage',
    version='0.4',
    description='loganalysispackage',
    long_description_content_type="text/markdown",
    long_description=README + '\n\n' + HISTORY,
    license='MIT',
    packages=find_packages(),
    author='Shinjini Ray',
    author_email='shinjini.ray@gmail.com',
    keywords=['log', 'analysis', 'neuralnetwork'],
    url='https://github.com/shinjiniray/loganalysispackage',
    download_url='https://pypi.org/project/loganalysispackage'
)

install_requires = [
	'pymongo',
	'dnspython',
	'elasticsearch',
	'numpy==1.18.5',
	'sklearn',
	'tensorflow',
	'matplotlib'
]

if __name__ == '__main__':
    setup(**setup_args, install_requires=install_requires)


import mongoconn as mc
from elasticsearchconn import getlogs


def classify(client, filePath):

    alllogs = getlogs(client, filePath)
    allinput = {}
    alloutput = {}
    alltimestamp = {}

    collection = mc.getMongoConn(filePath)
    log_details = collection.find({'client': client}, {'_id': 0})
    log_details = list(log_details)
    keywords = log_details[0]['keywords'].split(',')

    for logs in alllogs:
        input = []
        timestamp = []
        for i in range (0, len(alllogs[logs])):
            val = 0
            for word in keywords:
                if word.lower() in alllogs[logs][i]['message'].lower():
                    val = 1
            input.append(val)
            timestamp.append(alllogs[logs][i]['timestamp'])
        allinput[logs] = input
        alltimestamp[logs] = timestamp

    for logs in allinput:
        output = []
        for i in range (0, len(allinput[logs])):
            if allinput[logs][i] == 1:
                output.append(1)
            else:
                output.append(0)
        alloutput[logs] = output

    return allinput, alloutput, alltimestamp